/*
 * symfunc.cc - the "symmetric function coefficients" for the Virasoro
 * conditions
 *
 */

#include <stdlib.h>
#include <iostream>
#include <assert.h>

#include "symfunc.h"
#include "config.h"



// The database to store the numbers. It's opened automatically when
// used for the first time.

#ifdef CHECK_TIME
database SYMdbm((char*)"symfunc.dbm", (char*)DB_DIR, TIMESTAMP);
#else
database SYMdbm((char*)"symfunc.dbm", (char*)DB_DIR);
#endif



// Compute a symmetric function coefficient

rational symfunc (const rational& x, int k, int i)
{
	rational result;

	// Check the initial cases first.
	if ( (i<0) || (k<-1) || (i>k+1) ) return rational(0);
	if ( i==k+1 ) return rational(1);

	// Look up in the database
	SYMdbm.reset_key ();
	set_key (SYMdbm, x);
	set_key (SYMdbm, k);
	set_key (SYMdbm, i);
	if ( SYMdbm.read () ) {
		get_value (SYMdbm, result);
		return result;
	}

	// Not found, so compute it.
	result = 0;
	for (int j=0; j<=i; j++) result += (x+j)*symfunc (x+j+1, k-1-j, i-j);

	// Store it in the database.
	SYMdbm.reset_key ();
	SYMdbm.reset_value ();
	set_key (SYMdbm, x);
	set_key (SYMdbm, k);
	set_key (SYMdbm, i);
	set_value (SYMdbm, result);
	SYMdbm.write ();
	return result;
};

